from __future__ import unicode_literals

from mock import patch

from django.contrib.auth import get_user_model
from django.utils import timezone
from django.test import SimpleTestCase

from evotevotings.elections.models import (Election, Category, Candidate,
                                           ElectionCountQuery)


class ElectionTest(SimpleTestCase):
    """ """
    def test_str_equal_to_identifier(self):
        instance = Election(identifier="identifier")
        self.assertEqual(str(instance), instance.identifier)

    def test_is_list_mode(self):
        election = Election(presentation_mode=Election.PresentationMode.LIST)
        self.assertTrue(election.is_list_mode())
        election.presentation_mode = Election.PresentationMode.DROPDOWN
        self.assertFalse(election.is_list_mode())

    def test_is_dropdown_mode(self):
        election = Election(
            presentation_mode=Election.PresentationMode.DROPDOWN)
        self.assertTrue(election.is_dropdown_mode())
        election.presentation_mode = Election.PresentationMode.LIST
        self.assertFalse(election.is_dropdown_mode())

    def test_is_closed(self):
        election = Election(state=Election.State.CLOSED)
        self.assertTrue(election.is_closed)
        election.state = Election.State.OPEN
        self.assertFalse(election.is_closed)

    def test_is_in_definitive_eletoral_roll(self):
        election = Election(state=Election.State.DEFINITIVE_ELECTORAL_ROLL)
        self.assertTrue(election.is_in_definitive_eletoral_roll)
        election.state = Election.State.OPEN
        self.assertFalse(election.is_in_definitive_eletoral_roll)

    def test_is_in_to_vote(self):
        election = Election(state=Election.State.IN_TO_VOTE)
        self.assertTrue(election.is_in_to_vote)
        election.state = Election.State.OPEN
        self.assertFalse(election.is_in_to_vote)

    def test_can_manage_open(self):
        election = Election(state=Election.State.DEFINITIVE_ELECTORAL_ROLL)
        self.assertTrue(election.can_manage_open())
        election.state = Election.State.IN_TO_VOTE
        self.assertFalse(election.can_manage_open())

    def test_can_manage_close(self):
        election = Election(state=Election.State.DEFINITIVE_ELECTORAL_ROLL)
        self.assertTrue(election.can_manage_close())
        election.state = Election.State.IN_TO_VOTE
        self.assertTrue(election.can_manage_close())
        election.state = Election.State.CLOSED
        self.assertFalse(election.can_manage_close())

    def test_can_manage_counting(self):
        election = Election(state=Election.State.IN_COUNTING)
        self.assertTrue(election.can_manage_counting())
        election.state = Election.State.CLOSED
        self.assertTrue(election.can_manage_counting())
        election.state = Election.State.OPEN
        self.assertFalse(election.can_manage_counting())

    def test_can_manage_partial_counting(self):
        election = Election(state=Election.State.IN_TO_VOTE)
        self.assertTrue(election.can_manage_partial_counting())
        election.state = Election.State.CLOSED
        self.assertFalse(election.can_manage_partial_counting())

    @patch.object(Election, 'electoral_process')
    def test_can_change_with_action_configured(self, mock_fn):
        mock_fn.actions_configured = True
        election = Election(state=Election.State.DEFINITIVE_ELECTORAL_ROLL)
        self.assertTrue(election.can_change())
        election.state = Election.State.CLOSED
        self.assertFalse(election.can_change())

    @patch.object(Election, 'electoral_process')
    def test_can_change_with_action_unconfigured(self, mock_fn):
        mock_fn.actions_configured = False
        election = Election(state=Election.State.DEFINITIVE_ELECTORAL_ROLL)
        self.assertFalse(election.can_change())

    @patch.object(Election, 'can_change', return_value=True)
    def test_can_change_configs(self, mock_fn):
        election = Election()
        self.assertTrue(election.can_change_configs())

    @patch.object(Election, 'can_change', return_value=True)
    def test_can_change_calendar(self, mock_fn):
        election = Election()
        self.assertTrue(election.can_change_calendar())

    @patch.object(Election, 'can_change', return_value=True)
    def test_can_change_categories(self, mock_fn):
        election = Election(weight_votes=True)
        self.assertTrue(election.can_change_categories())
        election.weight_votes = False
        self.assertFalse(election.can_change_categories())

    @patch.object(Election, 'can_change', return_value=True)
    def test_cant_change_categories_in_definitive_eletoral_roll(self, mock_fn):
        election = Election(weight_votes=True,
                            state=Election.State.DEFINITIVE_ELECTORAL_ROLL)
        self.assertFalse(election.can_change_categories())

    @patch.object(Election, 'can_change', return_value=True)
    def test_can_change_rolls(self, mock_fn):
        election = Election()
        self.assertTrue(election.can_change_rolls())

    @patch.object(Election, 'can_change', return_value=True)
    def test_cant_change_rolls_in_definitive_eletoral_roll(self, mock_fn):
        election = Election(weight_votes=True,
                            state=Election.State.DEFINITIVE_ELECTORAL_ROLL)
        self.assertFalse(election.can_change_rolls())

    @patch.object(Election, 'can_change', return_value=True)
    def test_can_change_candidates(self, mock_fn):
        election = Election(proposition_end_date=timezone.now())
        self.assertTrue(election.can_change_candidates())
        election.proposition_end_date = timezone.now() + \
            timezone.timedelta(days=2)
        self.assertFalse(election.can_change_candidates())

    @patch.object(Election, 'candidates')
    @patch.object(Election, 'can_change', return_value=True)
    def test_can_change_bulletin_with_configs_filled(self, mock_fn,
                                                     mock_candidates):
        mock_candidates.exists.return_value = True
        election = Election(configs_filled=True)
        self.assertTrue(election.can_change_bulletin())
        mock_candidates.exists.return_value = False
        self.assertFalse(election.can_change_bulletin())

    @patch.object(Election, 'candidates')
    @patch.object(Election, 'can_change', return_value=True)
    def test_can_change_bulletin_with_configs_not_filled(self, mock_fn,
                                                         mock_candidates):
        mock_candidates.exists.return_value = True
        election = Election(configs_filled=True)
        self.assertTrue(election.can_change_bulletin())
        election.configs_filled = False
        self.assertFalse(election.can_change_bulletin())


class ElectionSignalsTest(SimpleTestCase):
    """ """

    def test_post_init(self):
        election = Election()
        self.assertTrue(election.is_tracking('roll_begin_date'))
        self.assertTrue(election.is_tracking('roll_end_date'))

    @patch('evotevotings.elections.models.publish_provisional_roll')
    def test_post_save_changes_for_roll_begin_date(self, mock_fn):
        election = Election(roll_begin_date=timezone.now())
        election.roll_begin_date = timezone.now() - timezone.timedelta(hours=1)
        Election.post_save_changes(Election, election)
        mock_fn.si.assert_called()
        mock_fn.si.assert_called_with(None)
        mock_fn.si.return_value.apply_async.assert_called()
        mock_fn.si.return_value.apply_async.assert_called_with(
            eta=election.roll_begin_date)

    @patch('evotevotings.elections.models.publish_definitive_roll')
    def test_post_save_changes_for_roll_end_date(self, mock_fn):
        election = Election(roll_end_date=timezone.now())
        election.roll_end_date = timezone.now() - timezone.timedelta(hours=1)
        Election.post_save_changes(Election, election)
        mock_fn.si.assert_called()
        mock_fn.si.assert_called_with(None)
        mock_fn.si.return_value.apply_async.assert_called()
        mock_fn.si.return_value.apply_async.assert_called_with(
            eta=election.roll_end_date)

    @patch.object(Election, 'categories')
    def test_post_save_create_category(self, mock_fn):
        election = Election()
        Election.post_save_create_category(Election, election, created=True)
        mock_fn.create.assert_called()
        mock_fn.create.assert_called_with(
            default=True, designation_en="One", designation_pt="Um")

    @patch("evotevotings.elections.models.initialize_blind_signatures",
           return_value=("test 1", "test 2"))
    def test_pre_save(self, mock_fn):
        election = Election()
        election._state.adding = True
        Election.pre_save(Election, election)
        mock_fn.assert_called()
        self.assertEqual(election.blindsig_init_components, "test 1")
        self.assertEqual(election.blindsig_pr_dash_components, "test 2")


class ElectionTranstitionsTest(SimpleTestCase):
    """ """
    # transitions conditions test

    @patch.object(Election, 'electoral_process')
    def test_can_publish(self, mock):
        mock.can_publish.return_value = True
        election = Election()
        self.assertTrue(election.can_publish())

    def test_can_publish_provisional_roll_with_date_surpassed(self):
        election = Election(roll_begin_date=timezone.now())
        self.assertTrue(election.can_publish_provisional_roll())

    def test_can_publish_provisional_roll_with_date_on_future(self):
        date = timezone.now() + timezone.timedelta(hours=1)
        election = Election(roll_begin_date=date)
        self.assertFalse(election.can_publish_provisional_roll())

    def test_can_publish_definitive_roll_with_date_surpassed(self):
        election = Election(roll_end_date=timezone.now())
        self.assertTrue(election.can_publish_definitive_roll())

    def test_can_publish_definitive_roll_with_date_on_future(self):
        date = timezone.now() + timezone.timedelta(hours=1)
        election = Election(roll_end_date=date)
        self.assertFalse(election.can_publish_definitive_roll())

    def test_can_begin_voting_with_date_surpassed(self):
        election = Election(voting_begin_date=timezone.now())
        self.assertTrue(election.can_begin_voting())

    def test_can_begin_voting_with_date_on_future(self):
        date = timezone.now() + timezone.timedelta(hours=1)
        election = Election(voting_begin_date=date)
        self.assertFalse(election.can_begin_voting())

    def test_can_end_voting_with_date_surpassed(self):
        election = Election(voting_end_date=timezone.now())
        self.assertTrue(election.can_end_voting())

    def test_can_end_voting_with_date_on_future(self):
        date = timezone.now() + timezone.timedelta(hours=1)
        election = Election(voting_end_date=date)
        self.assertFalse(election.can_end_voting())

    def test_can_close_with_date_surpassed(self):
        election = Election(minute_end_date=timezone.now())
        self.assertTrue(election.can_close())

    def test_can_close_with_date_on_future(self):
        date = timezone.now() + timezone.timedelta(hours=1)
        election = Election(minute_end_date=date)
        self.assertFalse(election.can_close())

    # transitions test

    @patch.object(Election, 'electoral_process')
    def test_publish_transition(self, mock_fn):
        mock_fn.can_publish.return_value = True
        election = Election(state=Election.State.NEW)
        election.publish()
        self.assertEqual(election.state, Election.State.OPEN)

    def test_publish_provisional_roll_transition(self):
        election = Election(state=Election.State.OPEN,
                            roll_begin_date=timezone.now())
        election.publish_provisional_roll()
        self.assertEqual(election.state,
                         Election.State.PROVISIONAL_ELECTORAL_ROLL)

    @patch('evotevotings.elections.models.ASrouter')
    def test_publish_definitive_roll_transition(self, mock_router):
        mock_router.return_value.initialize_election.return_value = True
        election = Election(state=Election.State.PROVISIONAL_ELECTORAL_ROLL,
                            roll_end_date=timezone.now())
        election.publish_definitive_roll()
        self.assertEqual(election.state,
                         Election.State.DEFINITIVE_ELECTORAL_ROLL)

    def test_voting_transition(self):
        election = Election(state=Election.State.DEFINITIVE_ELECTORAL_ROLL,
                            voting_begin_date=timezone.now())
        election.voting()
        self.assertEqual(election.state, Election.State.IN_TO_VOTE)

    def test_counting_transition(self):
        election = Election(state=Election.State.IN_TO_VOTE,
                            voting_end_date=timezone.now())
        election.counting()
        self.assertEqual(election.state, Election.State.IN_COUNTING)

    def test_scrutinized_transition(self):
        election = Election(state=Election.State.IN_COUNTING)
        election.scrutinized()
        self.assertEqual(election.state, Election.State.VOTES_SCRUTINIZED)

    def test_minute_in_approval_transition(self):
        election = Election(state=Election.State.VOTES_SCRUTINIZED)
        election.minute_in_approval()
        self.assertEqual(election.state, Election.State.MINUTE_IN_APPROVAL)

    def test_close_transition(self):
        election = Election(state=Election.State.MINUTE_IN_APPROVAL,
                            minute_end_date=timezone.now())
        election.close()
        self.assertEqual(election.state, Election.State.CLOSED)


class CategoryTest(SimpleTestCase):
    """ """

    def test_str_equal_to_designation(self):
        instance = Category(weight=2)
        self.assertEqual(str(instance), str(instance.weight))


class CandidateTest(SimpleTestCase):
    """ """

    def test_str_equal_to_designation(self):
        instance = Candidate(designation="designation")
        self.assertEqual(str(instance), instance.designation)


class ElectionCountQueryTest(SimpleTestCase):
    """ """

    def test_str_equal_to_expected_value(self):
        user = get_user_model()(name="test")
        election = Election(identifier="identifier")
        instance = ElectionCountQuery(
            election=election, user=user, total_count=10)
        expected = "{} ({}) - {}".format(
            election.identifier, user.name, instance.total_count)
        self.assertEqual(str(instance), expected)
