from __future__ import unicode_literals

from StringIO import StringIO
import json
from mock import patch

from django.core.urlresolvers import reverse

from contrib.umws.members.members_service import dummy_data

from. import ManagementTestCase


class SecureActionAjaxViewTest(ManagementTestCase):
    """ """

    fixture = ['actions.json']

    def setUp(self):
        self.login_as_member()
        self.member.electoral_process.electoral_actions.prepopulate()
        self.url = reverse("management:secureaction-validate", kwargs={
            "process_pk": self.member.electoral_process.pk,
            "pk": 1
        })

    def test_get(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/partials/secure_action.html")

    def test_get_unexistent_action(self):
        """ """
        url = reverse("management:secureaction-validate", kwargs={
            "process_pk": self.member.electoral_process.pk,
            "pk": 100
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_get_unexistent_process(self):
        """ """
        url = reverse("management:secureaction-validate", kwargs={
            "process_pk": 3,
            "pk": 1
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_post(self):
        """ """
        file = StringIO(b"Dummy file")
        data = {'key1': file}
        response = self.client.post(self.url, data=data)
        self.assertEqual(response.status_code, 200)
        response_json = json.loads(response.content)
        self.assertIn('errors_list', response_json)


class ManageSecureActionAjaxViewTest(ManagementTestCase):
    """ """

    fixture = ['actions.json']

    def setUp(self):
        # prepare a logged in member
        self.login_as_member()
        self.url = reverse("management:secureaction-manage", kwargs={
            "process_pk": self.member.electoral_process.pk,
        })

    def test_get(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/partials/secure_action.html")

    def test_get_unexistent_process(self):
        """ """
        url = reverse("management:secureaction-manage", kwargs={
            "process_pk": 3,
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_post(self):
        """ """
        file = StringIO(b"Dummy file")
        data = {"key1": file}
        response = self.client.post(self.url, data=data)
        response_json = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertIn('errors_list', response_json)


class ElectoralMemberVerifyViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_institutional()
        self.url = reverse("management:electoralmember-verify", kwargs={
            "process_pk": self.member.electoral_process.pk}) +\
            "?email=tfb@eurotux.com"

    @patch('evotevotings.management.forms.members_service.'
           'get_contact_info_by_email')
    def test_get(self, get_contact_info_by_email):
        """ """
        get_contact_info_by_email.return_value = dummy_data
        response = self.client.get(self.url)
        content = json.loads(response.content)
        result = False
        if 'name' in content and content['name'] == 'Dev Test':
            result = True
        self.assertEqual(response.status_code, 200)
        self.assertEqual(result, True)

    @patch('evotevotings.management.forms.members_service.'
           'get_contact_info_by_email')
    def test_get_error(self, get_contact_info_by_email):
        """ """
        url = reverse("management:electoralmember-verify", kwargs={
            "process_pk": self.member.electoral_process.pk}) +\
            "?email=unknown@eurotux.com"
        get_contact_info_by_email.return_value = {}
        response = self.client.get(url)
        content = json.loads(response.content)
        result = False
        if 'error' in content:
            result = True
        self.assertEqual(response.status_code, 200)
        self.assertEqual(result, True)


class AuthAmaAjaxViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_institutional()
        self.url = reverse("management:auth-ama", kwargs={
            "process_pk": self.member.electoral_process.pk
        })

    def test_template_used(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/partials/auth_ama.html")

    def test_login_required(self):
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class HistoryAjaxViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_member()
        self.url = reverse("management:electoralprocess-history-next", kwargs={
            "process_pk": self.member.electoral_process.pk
        })

    def test_template_used(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/includes/history_list.html")

    def test_entries_rendered(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertIn('No history', response.content)

    def test_login_required(self):
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class HistoryNewAjaxViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_member()
        self.url = reverse(
            "management:electoralprocess-history-count", kwargs={
                "process_pk": self.member.electoral_process.pk
            })

    def test_value_returned(self):
        """ """
        response = self.client.get(self.url)
        response_json = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertEqual({'new': 0}, response_json)

    def test_login_required(self):
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)
