from __future__ import unicode_literals

import json
import logging

from django.db.models import Count
from django.apps import apps
from django.core.exceptions import ObjectDoesNotExist
from rest_framework.serializers import ValidationError

from eVotUM.Cripto import pkiutils, eccblind, hashfunctions
from evotevotings.admin.models import SecretEntry
from evotevotings.admin.utils import (get_certificates,
                                      get_secret_entry,
                                      get_private_key_password)


logger = logging.getLogger('evote.logger')


def get_election_model():
    return apps.get_model("evoteelections.Election")  # noqa


def get_election_voter_model():
    return apps.get_model("evotevoters.ElectionVoter")  # noqa


def generate_blind_signature(signature=None, voter=None, election=None):
    """ Attempt to generate an hashed blind signature"""
    if (signature is None):
        raise ValidationError('Empty signature data')
    if (voter is None):
        raise ValidationError('Empty voter data')
    if (voter is None):
        raise ValidationError('Empty election data')
    try:
        blind_vote = signature['blind_hash']
        # signed_token = signature['signed_token']
        # Try to validate the signed token
        # in order to obtain the ElectionVoter and Election objects
        # (voter, election) = validate_vote_request(signed_token)

        # Get the election blind initComponents
        blind_components = election.blindsig_init_components

        # Generate the blind signature
        blind_signature = blind_vs_sign(blind_vote, blind_components)

        # Generate the blind signature hash
        blind_hash = hashfunctions.generateSHA256Hash(blind_signature)

        # Bind the blind signature hash to the ElectionVoter
        voter.blind_vote_hash = blind_hash

        # Set the voting status to PENDING
        ElectionVoter = get_election_voter_model()
        voter.vote_status = ElectionVoter.VoteStatus.PENDING

        # Save the model changes into the DB
        voter.save()

        # Return the blind signature to the requestor
        return blind_signature

    except ValidationError:
        # TODO: Log and handle ValidationError
        return None


def validate_vote_request(signature=None):
    """ Validate the necessary request data """
    try:
        if (signature is None):
            raise ValidationError('Empty signed token')

        Election = get_election_model()
        ElectionVoter = get_election_voter_model()

        # Validate the certificates
        certs = get_certificates()

        # Missing voting system certificate
        if (not certs['votesys']):
            raise ValidationError(
                'Missing voting system certificate from the DB'
            )

        # Missing root certificate
        if (not certs['root']):
            raise ValidationError(
                'Missing root certificate from the DB'
            )

        # Check if voting system certificate is chained to root
        chain = pkiutils.verifyCertificateChain(
                                        certs['votesys'],
                                        certs['root'])
        if(not chain):
            raise ValidationError('Unchained voting system certificate')

        # Validate voting system certificate expiration
        (error_code, pubkey) = pkiutils.getPublicKeyFromCertificate(
                                                    certs['root'])

        if (error_code is not None):
            raise ValidationError(
                'Expired voting system certificate. ' +
                '(getPublicKeyFromCertificate) Error:' +
                str(error_code))

        # Avoid Base64 padding errors
        signature = str(signature)

        # Validate signed object
        (error_code, unsigned_obj) = pkiutils.verifyObjectSignature(
                                        signature,
                                        certs['votesys'])

        if (error_code is not None):
            raise ValidationError(
                'Could not validate object signature. ' +
                '(verifyObjectSignature) Error:' +
                str(error_code))

        unsigned_obj = json.loads(unsigned_obj)

        # Validate Election Period
        election_id = unsigned_obj['election_id']
        if election_id is None:
            raise ValidationError('Unsigned object contains no election_id')

        election_obj = Election.objects.get(pk=election_id)
        if not election_obj.is_in_to_vote:
            raise ValidationError(
                'Election {'+election_id+'} not in voting period')

        # Validate user vote capability
        user_id = unsigned_obj['user_id']
        if user_id is None:
            raise ValidationError('Unsigned object contains no user_id')

        voter = ElectionVoter.objects.get(user_id=user_id,
                                          election_id=election_id)

        if(election_obj.pk != voter.election.pk):
            raise ValidationError('User cannot vote in this election')

        (code, vote_available) = voter.is_vote_available
        if(not vote_available):
            raise ValidationError(
                'Vote not available for this user. ' +
                'Vote Status:' + str(code))

        # Return the ElectionVoter and Election objects pair
        return (voter, election_obj)

    except ValidationError as e:
        # Raise exception to the top level
        logger.critical(format(e))
        raise e


def blind_vs_sign(blind_data=None, init_components=None):
    """ Sign blind data using the validator service private key """
    try:
        if (blind_data is None):
            raise ValidationError('Empty blind_data to sign')

        if (init_components is None):
            raise ValidationError('Empty init_components for the blind_data')

        # Get the Validator Service Private Key
        vs_private_key = get_secret_entry(
                                SecretEntry.Identifier.VALIDATOR_PVT_KEY
                     )

        # Get the Validator Service Private Key Password
        pvt_key_password = get_private_key_password()

        # Atempt to generate the blind signature
        (error_code, signature) = eccblind.generateBlindSignature(
                                        vs_private_key,
                                        pvt_key_password,
                                        blind_data,
                                        init_components)

        # Check if blind_data was successfully signed
        if(error_code is not None):
            raise ValidationError(
                'Could sign the blind data. ' +
                '(generateBlindSignature) Error:' +
                str(error_code))

        # Return the blind signature
        return signature

    except ValidationError as e:
        logger.critical(format(e))
        raise e


def unsign_election_id(signature):
    """ Try to unsign the election_id using the
        voting system certificate """
    if (signature is None):
        return False

    certs = get_certificates()

    # Missing voting system certificate
    if (not certs['votesys']):
        logger.critical("No Validator Service certificate")
        return False

    # Validate signed object
    (error_code, unsigned_obj) = pkiutils.verifyObjectSignature(
                                        signature,
                                        certs['votesys'])

    if(error_code is not None):
        return unsigned_obj
    else:
        logger.critical(
            "(%d): Could not verify object signature" % error_code)

    return False


def check_vote_status(blind_hash, election_id):
    ElectionVoter = get_election_voter_model()
    try:
        election_voter = ElectionVoter.objects.get(
                blind_vote_hash=blind_hash,
                election=election_id
                )
        (status, avail) = election_voter.is_vote_available
        if(status == ElectionVoter.VoteStatus.PENDING):
            election_voter.vote_status = ElectionVoter.VoteStatus.YES
            election_voter.save()
            return True
        else:
            return False

    except ObjectDoesNotExist:
        return False


def vote_status_results(election_id):
    Election = get_election_model()
    vote_status_list = {}

    election = Election.objects.get(pk=election_id)
    totals = election.election_voters.all()\
        .values('vote_status')\
        .annotate(total=Count('vote_status'))

    for result in totals:
        idx = str(result['vote_status'])
        vote_status_list[idx] = str(result['total'])

    return vote_status_list
