from __future__ import unicode_literals

import json
import requests

from django.utils import timezone
from django.core.management.base import BaseCommand
from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse

from evotevotings.elections.utils import get_extra_election_data
from evotevotings.elections.utils import initialize_blind_signatures
from evotevotings.admin.models import SecretEntry
from evotevotings.voters.models import ElectionVoter
from evotevotings.elections.models import Election
from evotevotings.processes.models import ElectoralProcess
from evotevotings.elections.utils import get_secret_entry
from evotevotings.services.routers import ASrouter
from eVotUM.Cripto import hashfunctions, pkiutils, eccblind
from evotelogs.registers import audit_register


PROCESS_NUM = 10
BULK_BATCH_LIMIT = 1500


class Command(BaseCommand):
    help = 'Inserts the requested number of votes into an arbitrary election'
    # Class variables
    # self.election : Election created to simulate the votes

    def add_arguments(self, parser):
        parser.add_argument(
            'num_voters', nargs='?', help='Number of votes to simulate',
        )
        parser.add_argument(
            'base_url', nargs='?', help='DNS to generate the request',
        )

    def handle(self, *args, **options):
        audit_register.remove(ElectionVoter)
        audit_register.remove(get_user_model())
        # Set number of votes to simulate
        num_voters = int(options.get('num_voters', 10))
        # Set the base DNS
        self.base_url = options.get('base_url', 'http://localhost:8000')
        self.iterator = 1
        self.max_votes = num_voters
        # Create the necessary data
        try:
            self.generate_election(num_voters)
            self.print_success("Finished generating election data\n")
            # Vote with each voter
            for voter in self.voters:
                self.vote(voter)
                self.iterator += 1
        except Exception as e:
            print e

        # Cleanup
        self.voters.delete()
        self.users.delete()
        self.election.categories.all().delete()
        self.election.delete()
        self.electoral_process.delete()

    def vote(self, voter):
        """Simulate a vote for a generated user"""
        step_zero_data = get_extra_election_data(
            self.election.pk,
            voter.user_id
        )
        # SHA256 combo_id
        combo_hash = hashfunctions.generateSHA256Hash(
            step_zero_data['combo_id']
        )
        # Get the JSON vote
        json_vote = self.generate_json_vote()
        # Encrypt with election certificate
        elect_cert = self.election.certificate
        ciphered_vote = pkiutils.encryptWithPublicKey(json_vote, elect_cert)
        if ciphered_vote is None:
            self.print_error("Could not cipher the vote")
            return False
        ciphered_vote_hash = hashfunctions.generateSHA256Hash(
            ciphered_vote
        )
        final_hash = combo_hash + ciphered_vote_hash
        final_hash = hashfunctions.generateSHA256Hash(final_hash)

        pr_components = self.election.blindsig_pr_dash_components
        (err, blind_data) = eccblind.blindData(pr_components, ciphered_vote)

        if err is not None:
            self.print_error("Could not generate blind vote data")
            return False

        new_blind_components = blind_data[0]
        new_pr_components = blind_data[1]
        blind_vote = blind_data[2]

        combo_id = step_zero_data['combo_id']

        data_to_validate = {
            'signed_token': combo_id,
            'blind_hash': blind_vote,
            'pin_value': 'abcd'
        }
        blind_signature = self.validator_service(data_to_validate)
        if blind_signature is None:
            return False

        hashed_blind = hashfunctions.generateSHA256Hash(
            str(blind_signature)
        )

        (err, signature) = eccblind.unblindSignature(
            str(blind_signature),
            str(pr_components),
            str(new_blind_components)
        )
        if err is not None:
            self.print_error("(%d) Could not unblind signature" % err)
            return False

        (err, valid) = eccblind.verifySignature(
            step_zero_data['vs_cert'],
            str(signature),
            new_blind_components,
            new_pr_components,
            ciphered_vote
        )
        if err is not None:
            self.print_error("(%d) Invalid blind signature" % err)
            return False

        data_to_filter = {
            'election_id': step_zero_data['election_id'],
            'blind_hash': hashed_blind,
            'ciphered_vote': ciphered_vote,
            'signed_vote': signature,
            'blind_components': new_blind_components,
            'pr_components': new_pr_components,
            'unique_bulletin': final_hash
        }
        vote_veredict = self.filter_service(data_to_filter)
        if vote_veredict:
            self.print_success(
                "[%d/%d] Successfully inserted vote: %s" %
                (self.iterator, self.max_votes, final_hash)
            )
        else:
            self.print_error(
                "[%d/%d] Failed to insert vote" %
                (self.iterator, self.max_votes)
            )

    def filter_service(self, request_data):
        url = self.base_url + reverse('genericservice:filtersignature')
        request_data = json.dumps(request_data)
        response = requests.post(
            url,
            data=request_data,
            headers={'content-type': 'application/json'},
            timeout=None
        )
        if response.status_code != 200:
            print response.content
            self.print_error('Could not insert vote')
            return None
        json_response = response.json()
        return json_response.pop("result", False)

    def validator_service(self, request_data):
        url = self.base_url + reverse('genericservice:validatevote')
        request_data = json.dumps(request_data)
        response = requests.post(
            url,
            data=request_data,
            headers={'content-type': 'application/json'},
            timeout=None
        )
        if response.status_code != 200:
            print response.content
            self.print_error('Could not validate vote request')
            return None
        json_response = response.json()
        return json_response.pop('data', None)

    def generate_json_vote(self):
        vote = [
            {
                'blank': False,
                'candidates': [
                    {
                        'id': 1,
                        'vote': 1
                    }
                ],
                'category': 'one'
            }
        ]
        json_vote = json.dumps(vote)
        return json_vote

    def generate_election(self, num_voters=10):
        """"""
        # Vote Status
        status = Election.State.IN_TO_VOTE

        now = timezone.now()

        self.electoral_process = ElectoralProcess(
            created=now,
            created_by_id=1,
            identifier='Dummy Electoral Process',
        )
        self.electoral_process.save()

        # Generate Election data
        # Using the voting system certificate as election certificate
        votesys_cert_id = SecretEntry.Identifier.VOTING_CERT
        # Blind signature components
        (init_component, pr_component) = initialize_blind_signatures()
        self.election = Election(
            blindsig_init_components=init_component,
            blindsig_pr_dash_components=pr_component,
            identifier='Dummy Election',
            created=now,
            state=status,
            certificate=get_secret_entry(votesys_cert_id),
            proposition_begin_date=now,
            proposition_end_date=now,
            roll_begin_date=now,
            roll_end_date=now,
            voting_begin_date=now,
            voting_end_date=now,
            minute_begin_date=now,
            minute_end_date=now,
            minute_publication_date=now,
            electoral_process=self.electoral_process
        )
        self.election.save()
        voters = []
        users = []

        mail_suffix = 'simulate.vote.org'
        category_id = self.election.categories.only('pk').first().pk

        # Iterate through the number of voters requested
        for i in range(num_voters):
            user = get_user_model()(
                created=now,
                email="dummy%d@%s" % (i, mail_suffix),
                name="dummy%d" % i,
            )
            users.append(user)

            if(len(users) > BULK_BATCH_LIMIT):
                get_user_model().objects.bulk_create(users)
                users = []

        get_user_model().objects.bulk_create(users)
        self.users = get_user_model().objects\
            .filter(email__contains=mail_suffix)

        for user_id in self.users.values_list('pk', flat=True):
            voter = ElectionVoter(
                name=mail_suffix,
                created=now,
                election=self.election,
                user_id=user_id,
                category_id=category_id,
                token_expiration_date=now + timezone.timedelta(days=1),
                pin_value='abcd',
                token_validated=True
            )
            voters.append(voter)

            if(len(voters) > BULK_BATCH_LIMIT):
                ElectionVoter.objects.bulk_create(voters)
                voters = []

        ElectionVoter.objects.bulk_create(voters)

        # Populate the needed querysets
        self.voters = ElectionVoter.objects.filter(name=mail_suffix)

        # Initialize Election in the AS
        as_router = ASrouter()
        as_router.initialize_election(self.election.pk, num_voters)
        self.print_success("Initialized election with %d entries" % num_voters)

    def print_success(self, msg):
        self.stdout.write(self.style.SUCCESS(msg))

    def print_error(self, msg):
        self.stderr.write(self.style.ERROR(msg))
