from __future__ import unicode_literals

import datetime
from eVotUM.Cripto.shamirsecret import generateSecretTime

from django.contrib.auth.decorators import login_required
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse_lazy, reverse
from django.utils.decorators import method_decorator
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.views.generic import ListView, DetailView, TemplateView
from django.shortcuts import get_object_or_404
from django.utils import timezone
from django.conf import settings

from evotevotings.processes.models import ElectoralProcess
from evotevotings.elections.models import Election
from evotevotings.elections.utils import get_extra_election_data
from evoteajaxforms.views import FormJsonSimpleView, AjaxResponseAction
from evotemessages.mixins import MessageBaseMixin, MessageFormBaseMixin
from evotecore.decorators import search_register
from evoteajaxforms.views import FormJsonView
from evotemessages.models import Message
from evotecore.mixins import (DjangoTablesFilterMixin, PermissionListMixin,
                              PermissionRequiredMixin, SelectedMenuMixin)
from evoteusers.views import (EvoteLoginView, PersonalDataBaseView,
                              ValidateEmailBaseView,
                              ResendPendingTokenMailBaseView)
from contrib.umws.sms import sms_service
from evotecore.views import ErrorBaseView

from .middleware import SESSION_USER_PROFILE
from .mailings import SendPinMail
from .mailings import ContactMail
from .filters import (ArchiveFilter, ElectionHistoryFilter, IndexFilter,
                      VoterFilter)
from .tables import (ElectoralProcessTable, ElectoralProcessArchiveTable,
                     ElectoralProcessMyTable, ElectionHistoryTable,
                     ElectionVoterTable)
from .models import ElectionVoter, ElectoralRoll
from .forms import (ElectionVotingStep1Form, ElectionVotingStep3Form,
                    ElectionVotingStep3PinForm, ElectionSearchVoteForm,
                    ContactsForm)
from .utils import get_references
from .menus import Menu


class LoginView(EvoteLoginView):
    """Provides user the ability to login"""
    template_name = "voters/login.html"

    def form_valid(self, form):
        self.request.session[SESSION_USER_PROFILE] = \
            form.cleaned_data.get('profile')
        return super(LoginView, self).form_valid(form)


class ElectoralProcessListView(
        SelectedMenuMixin, PermissionListMixin, DjangoTablesFilterMixin,
        ListView):
    """
    Shows the main view
    """
    model = ElectoralProcess
    queryset = ElectoralProcess.objects.published()
    permission_required = "evoteprocesses.view_electoralprocess"
    # tables
    table_pagination = {'per_page': 5}
    # filters
    filter_class = IndexFilter

    def get_table_class(self):
        if self.request.user.is_authenticated():
            return ElectoralProcessMyTable
        return ElectoralProcessTable

    def get_template_names(self):
        if self.request.user.is_authenticated():
            return ["voters/electoralprocess_mylist.html"]
        return ["voters/electoralprocess_list.html"]


class ElectoralProcessDetailView(
        SelectedMenuMixin, PermissionRequiredMixin, DetailView):
    """
    Shows the Electoral Process Detail view
    """
    model = ElectoralProcess
    permission_required = "evoteprocesses.view_electoralprocess"

    def get_template_names(self):
        if self.request.user.is_authenticated():
            return ["voters/electoralprocess_mydetail.html"]
        return ["voters/electoralprocess_detail.html"]

    def get_context_data(self, **kwargs):
        context = super(ElectoralProcessDetailView, self)\
            .get_context_data(**kwargs)
        context.update({
            'show_vote_link': self.object.elections.eligible(self.request.user)
            .in_to_vote().exists(),
            'state_scrutinized': Election.State.VOTES_SCRUTINIZED
        })
        return context


class ElectoralProcessArchiveView(
        SelectedMenuMixin, PermissionListMixin, DjangoTablesFilterMixin,
        ListView):
    """
    Shows the archive view
    """
    model = ElectoralProcess
    queryset = ElectoralProcess.objects.closed()
    permission_required = "evoteprocesses.view_electoralprocess"
    template_name = "voters/electoralprocess_archive.html"
    # tables
    table_class = ElectoralProcessArchiveTable
    table_pagination = {'per_page': 5}
    # filters
    filter_class = ArchiveFilter


@method_decorator(login_required, name="dispatch")
class ElectionHistoryView(
        SelectedMenuMixin, PermissionListMixin, DjangoTablesFilterMixin,
        ListView):
    """
    Shows the my elections history view
    """
    model = Election
    queryset = Election.objects.closed()
    permission_required = "evoteelections.view_election"
    template_name = "voters/election_history.html"
    # tables
    table_class = ElectionHistoryTable
    table_pagination = {'per_page': 5}
    # filters
    filter_class = ElectionHistoryFilter
    selected_menu = Menu.HISTORICAL


@method_decorator(login_required, name="dispatch")
class ElectoralProcessVotingView(
        SelectedMenuMixin, PermissionRequiredMixin, DetailView):
    """
    Shows the voting system
    """
    model = ElectoralProcess
    permission_required = "evoteprocesses.view_electoralprocess"
    template_name = "voters/electoralprocess_voting.html"

    def get_context_data(self, **kwargs):
        """ """
        context = super(ElectoralProcessVotingView, self)\
            .get_context_data(**kwargs)
        elections = self.object.elections.eligible(self.request.user)\
            .in_to_vote()
        context.update({
            'elections': elections,
            'elections_active_count': elections.count(),
            'close_url': reverse_lazy(
                "voters:electoralprocess-detail", kwargs={
                    'slug': self.object.slug
                })
        })
        return context


class ElectionVotingStepMixin(PermissionRequiredMixin):
    """
    Generic class for all steps
    """
    model = Election
    permission_required = "evoteelections.vote_election"
    slug_url_kwarg = "slug_election"

    def get_context_data(self, **kwargs):
        context = super(ElectionVotingStepMixin,
                        self).get_context_data(**kwargs)
        context.update({
            'close_url': reverse_lazy(
                "voters:electoralprocess-detail", kwargs={
                    'slug': self.object.electoral_process.slug
                })
        })
        return context


@method_decorator(login_required, name="dispatch")
class ElectionVotingStep1View(ElectionVotingStepMixin, DetailView):
    """
    Shows the voting system (step 1)
    """
    model = Election
    form_class = ElectionVotingStep1Form
    template_name = "voters/election_voting_step1.html"

    def get_context_data(self, **kwargs):
        context = super(ElectionVotingStep1View, self)\
            .get_context_data(**kwargs)

        form_kwargs = {'instance': self.object}
        self.elect_data = get_extra_election_data(self.object.pk,
                                                  self.request.user.pk)
        if self.elect_data is not False:
            form_kwargs.update(self.elect_data)
            combo_id = self.elect_data['combo_id']
        else:
            combo_id = False

        context.update({
            'form': self.form_class(**form_kwargs),
            'is_valid': bool(self.elect_data),
            'combo_id': combo_id
        })
        return context


@method_decorator(login_required, name="dispatch")
class ElectionVotingStep2View(ElectionVotingStepMixin, DetailView):
    """
    Shows the voting system (step 2)
    """
    template_name = "voters/election_voting_step2.html"


@method_decorator(login_required, name="dispatch")
class ElectionVotingStep3View(ElectionVotingStepMixin, FormJsonView):
    """
    Shows the voting system (step 3)
    """
    model = Election
    form_class = ElectionVotingStep3Form
    template_name = "voters/election_voting_step3.html"

    def get_context_data(self, **kwargs):
        context = super(ElectionVotingStep3View, self)\
            .get_context_data(**kwargs)

        context.update({
            'succ_url': self.get_success_url(),
            'error_url': self.request.get_full_path()
        })
        return context

    def get_form_kwargs(self):
        """ """
        form_kwargs = super(ElectionVotingStep3View, self).get_form_kwargs()
        form_kwargs.update({'user': self.request.user})
        return form_kwargs

    def form_valid(self, form):
        """ Save the PIN value, pin expiration date and send pin through info
            in election voter table """
        send_through = form.cleaned_data['send_through']
        ev = get_object_or_404(ElectionVoter,
                               user=self.request.user.pk,
                               election=self.object.pk)

        pin_ttl = getattr(settings, 'PIN_TIMETOLIVE', 120)
        (secret, expiration) = generateSecretTime(settings.PIN_SIZE,
                                                  pin_ttl)
        expiration_date = timezone.now() \
            + datetime.timedelta(0, pin_ttl)
        ev.pin_value = secret
        ev.token_send_through = send_through
        ev.token_expiration_date = expiration_date
        ev.token_validated = False
        ev.save(update_fields=['pin_value',
                               'token_validated',
                               'token_send_through',
                               'token_expiration_date'])
        if send_through == 'sms':
            send_to = self.request.user.get_phone()
            smsid = sms_service.send_message(send_to, secret)
            print "Pin SMS id:", smsid  # TODO log
        else:
            user_name = self.request.user.name
            send_to = self.request.user.get_email()
            mail_info = {'pin_value': secret,
                         'pin_expiration_date': expiration_date}
            SendPinMail(mail_info).send([(user_name, send_to)])

        return super(ElectionVotingStep3View, self).form_valid(form)

    def get_success_url(self):
        """ On success redirect to next step """
        return reverse_lazy("voters:election-voting-step3-pin", kwargs={
            'slug': self.kwargs.get('slug'),
            'slug_election': self.kwargs.get('slug_election')
        })


@method_decorator(login_required, name="dispatch")
class ElectionVotingStep3PinView(ElectionVotingStepMixin, FormJsonView):
    """
    Shows the voting system (step 3 - PIN)
    """
    model = Election
    form_class = ElectionVotingStep3PinForm
    template_name = "voters/election_voting_step3_pin.html"

    def get_election_voter(self):
        return get_object_or_404(ElectionVoter, user=self.request.user.pk,
                                 election=self.object.pk)

    def get_context_data(self, **kwargs):
        context = super(ElectionVotingStep3PinView, self)\
            .get_context_data(**kwargs)

        self.ev = self.get_election_voter()

        saml_confs = getattr(settings, 'SAML', '')
        sess_key = saml_confs.get('session_consume', None)
        if sess_key in self.request.session:
            # Set the vote token as valid
            ttl = getattr(settings, 'PIN_TIMETOLIVE', 120)
            self.ev.token_send_through = sess_key
            expiration_date = timezone.now() \
                + datetime.timedelta(0, ttl)
            self.ev.token_expiration_date = expiration_date
            self.ev.token_validated = True
            self.ev.save(update_fields=['token_validated',
                                        'token_send_through',
                                        'token_expiration_date'])

        # Check if token is valid
        authenticated = bool(self.ev.token_validated)
        context.update({
            'authenticated': authenticated
        })
        return context


@method_decorator(login_required, name="dispatch")
class ElectionVotingStep4View(ElectionVotingStepMixin, DetailView):
    """
    Shows the voting system (step 4)
    """
    template_name = "voters/election_voting_step4.html"


@method_decorator(login_required, name="dispatch")
class ElectionSearchVoteView(PermissionRequiredMixin, DetailView):
    """
    Seach vote in some election
    """
    model = Election
    form_class = ElectionSearchVoteForm
    permission_required = "evoteelections.vote_election"
    template_name = "voters/election_search_vote.html"
    slug_url_kwarg = "slug_election"

    def get_context_data(self, **kwargs):
        """ """
        context = super(ElectionSearchVoteView, self).\
            get_context_data(**kwargs)

        page = self.request.GET.get('page', 1)
        election_id = self.object.pk
        electoral_process_id = self.object.electoral_process_id
        search_text = self.request.GET.get('search') or ''

        (references, more) = get_references(election_id,
                                            page,
                                            search_text)

        context.update({
            'form': self.form_class(self.request.GET),
            'references': references,
            'search': search_text,
            'election_id': election_id,
            'electoral_process_id': electoral_process_id,
            'page': page,
            'next_page': int(page) + 1,
            'more': more
        })

        return context


class ContactsView(SelectedMenuMixin, FormJsonSimpleView):
    """ """
    prefix = 'contacts'
    form_class = ContactsForm
    template_name = "voters/contacts.html"
    action = AjaxResponseAction.REFRESH
    selected_menu = Menu.CONTACTS
    success_message = _("Your request was submitted successfully.")

    def form_valid(self, form):
        context = {
            'name': form.cleaned_data.get('name', ''),
            'from': form.cleaned_data.get('email', ''),
            'subject': form.cleaned_data.get('subject', ''),
            'message': form.cleaned_data.get('message', '')
        }
        ContactMail(context).send()
        return super(ContactsView, self).form_valid(form)

    def get_context_data(self, **kwargs):
        """ """
        context = super(ContactsView, self).get_context_data(**kwargs)

        context['contact_name'] = getattr(
            settings, "EVOTE_CONTACT_NAME", None)
        email = getattr(
            settings, "EVOTE_CONTACT_MAIL", None)
        if email:
            context['contact_mail'] = email[0][1]
        context['contact_phone'] = getattr(
            settings, "EVOTE_CONTACT_PHONE", None)
        context['contact_address'] = getattr(
            settings, "EVOTE_CONTACT_ADDRESS", None)
        context['contact_map_address'] = getattr(
            settings, "EVOTE_CONTACT_ADDRESS", False)
        context['contact_map_gps'] = getattr(
            settings, "EVOTE_CONTACT_GPS",
            {'lat': 41.5598965, 'lng': -8.3988118})
        return context


@method_decorator(login_required, name="dispatch")
class PersonalDataView(SelectedMenuMixin, PersonalDataBaseView):
    """ """
    template_name = "voters/personal_data.html"
    selected_menu = Menu.PERSONALDATA

    def get_form_kwargs(self):
        """ """
        form_kwargs = super(PersonalDataView, self).get_form_kwargs()
        form_kwargs.update({
            'base_url': self.request.build_absolute_uri(
                reverse("voters:validate-email"))
        })
        return form_kwargs

    def get_context_data(self, **kwargs):
        context = super(PersonalDataView, self).get_context_data(**kwargs)
        context.update({
            'resend_url': reverse_lazy("voters:resend-email")
        })
        return context


class ValidateEmailView(ValidateEmailBaseView):
    """ """
    template_name = "voters/validate_email.html"


@method_decorator(login_required, name="dispatch")
class ResendPendingTokenMailView(ResendPendingTokenMailBaseView):
    """ Resends email with token """

    def get_redirect_url(self):
        return reverse_lazy("voters:personal-data")

    def get_base_validate_url(self):
        return reverse("voters:validate-email")


class SearchView(TemplateView):
    """ View showing search results from the base search query """
    template_name = "voters/search.html"

    def get_context_data(self, **kwargs):
        """  """
        context = super(SearchView, self).get_context_data(**kwargs)
        # Search the models based on the GET 'q' parameter value
        # as query string
        query_string = self.request.GET.get('q', "")

        # Only search if a valid query string exists.
        if query_string != "":

            # Split each word of the querystring into a list
            terms = query_string.split()

            results = []

            # Iterate over the search register in order to detect all the
            # classes that have the "searchable" annotation.
            for r in search_register:
                results += r.search(terms)

            # Prepare pagination
            page = self.request.GET.get('page')
            # Define pagination (and number of results p/ page)
            paginator = Paginator(results, 10)
            try:
                results = paginator.page(page)
            except PageNotAnInteger:
                # If not a valid page param, return the first page.
                results = paginator.page(1)
            except EmptyPage:
                # If page out of range, return last
                results = paginator.page(paginator.num_pages)

            context['query'] = query_string
            context['terms'] = terms
            context['results'] = results

        return context


class ElectionClaimView(
        SelectedMenuMixin, MessageBaseMixin, MessageFormBaseMixin,
        FormJsonSimpleView):
    """ """
    prefix = 'claimform'
    template = "evotemessages/message_form.html"
    action = AjaxResponseAction.REDIRECT
    success_message = _("Your claim was sent.")
    claim = True

    def get_success_url(self):
        return reverse_lazy(
            "voters:electoralprocess-detail", args=[self.kwargs['slug']])

    def form_valid(self, form):
        context = self.get_context_data()
        instance = form.save()
        process = context['process']
        instance.content_object = process
        instance.save()
        instance.recipients.add(
            *[member.user for member in process.electoral_members.all()])
        name = form.cleaned_data.get('name')
        Message.objects.notify_anonymous_sender(instance, name)
        Message.objects.send(instance, name)
        self.set_success_message()
        return self.json_to_response()

    def get_initial(self):
        initial = super(ElectionClaimView, self).get_initial()
        election = get_object_or_404(
            Election, slug=self.kwargs['slug_election'])
        initial.update({
            'subject': _('Claim - Election ') + election.identifier
        })
        if self.request.user.is_authenticated():
            initial.update({
                'name': self.request.user.name,
                'email': self.request.user.get_email()
            })
        return initial

    def get_context_data(self, **kwargs):
        context = super(ElectionClaimView, self).get_context_data(**kwargs)
        context.update({
            'process': get_object_or_404(
                ElectoralProcess, slug=self.kwargs.get('slug'))
        })
        return context


class HowitworksView(SelectedMenuMixin, TemplateView):
    """ """
    template_name = "voters/howitworks.html"
    selected_menu = Menu.HOWDOESITWORK


class ElectoralRollDetailView(
        SelectedMenuMixin, DjangoTablesFilterMixin, DetailView):
    """ """
    model = ElectoralRoll
    table_class = ElectionVoterTable
    template_name = "voters/electoralroll_detail.html"
    slug_url_kwarg = "slug_roll"
    filter_class = VoterFilter

    def get_table_data(self):
        """ """
        table_filter = self.get_filter(self.object.election_voters.all())
        return table_filter.qs


# Error views
class VotersErrorView(ErrorBaseView):
    template_name = 'voters/errors/base.html'
